#######################################################################
#
# NCM component for autofs configuration
#
# David Groep <davidg@nikhef.nl>
# Version 0.2.1 : 28/12/04 18:50
#
# ** Generated file : do not edit **
#
#######################################################################

package NCM::Component::autofs;

use strict;
use NCM::Component;
use LC::Process;
use vars qw(@ISA $EC);
@ISA = qw(NCM::Component);
$EC=LC::Exception::Context->new->will_store_all;
use NCM::Check;

use File::Path;

use EDG::WP4::CCM::Element;


local(*DTA);

##########################################################################
sub getValueDefault() {
##########################################################################
    my ($config, $pathname, $default) = @_;

    if ($config->elementExists($pathname)) {
        return $config->getValue($pathname);
    } else {
      return $default;
    }
}


##########################################################################
sub writeAutoMap($$@) {
##########################################################################
    my ($self,$config,$mapname,$cfpathname) = @_;
              
    my ($changes) = (0);
              
    $self->debug(2,"Updating $mapname");

    stat $mapname or do {
      local(*TMPH);
      open TMPH,">$mapname" and close TMPH;
    };  
    LC::Check::status("$mapname", owner=> "root", group=>"root", mode=>0644);

    my $entrylist = $config->getElement("$cfpathname");
    while ( $entrylist->hasNextElement() ) {
      my $entry=$entrylist->getNextElement()->getName();

      my $opt = &getValueDefault($config,"$cfpathname/$entry/options","");
      my $location = &getValueDefault($config,"$cfpathname/$entry/location","");
      if ( $location eq "" ) {
        $self->warn("Location for entry $entry in $mapname is empty,".
                    " ignoring map");
        return 0;
      }

      $entry=~s/__wildcard/\*/;

      my $reentry=$entry; $reentry=~s/\*/\\\*/;
      $changes+=NCM::Check::lines( $mapname,
              linere => "^#?$reentry\\s+.*",
              goodre => "^$reentry\\s+$opt\\s+$location\$",
              good   => "$entry\t$opt\t$location",
              keep   => "first",
              add    => "last" );
    }
    $changes and
        $self->info("Automount map $mapname modified, $changes updates");
    return 1;
}

##########################################################################
sub Configure($$@) {
##########################################################################
    
    my ($self, $config) = @_;

    # Define paths for convenience. 
    my $base = "/software/components/autofs";
    my $cnt  = 0;

    if ( $config->elementExists("$base/maps") ) {

      my $maps = $config->getElement("$base/maps");
      while ( $maps->hasNextElement() ) {
        my $map=$maps->getNextElement()->getName();

        my @mountpoints;
        $config->elementExists("$base/maps/$map/mountpoint") and
          push @mountpoints,$config->getValue("$base/maps/$map/mountpoint");

        if ( $config->elementExists("$base/maps/$map/mpaliases") ) {
          $self->warn("Using depricated mpaliases functionality for $map");
          my $mpaliases = $config->getElement("$base/maps/$map/mpaliases");
          while ( $mpaliases->hasNextElement() ) {
            push @mountpoints,$mpaliases->getNextElement()->getValue();
          }
        }
        $mountpoints[0] or $mountpoints[0]="/$map";

        my $maptype=$config->getValue("$base/maps/$map/type");
        my $mapname;
        my $mpopts;
        if ( $config->elementExists("$base/maps/$map/mapname") ) {
          $mapname=$config->getValue("$base/maps/$map/mapname");
        } else { # we need to guess a mapname
          foreach ( $maptype ) {
          /file/ and $mapname="/etc/auto.$map";
          /yp/   and $mapname="auto.$map";
          }
        }
        $mapname or $self->error("Cannot figure out mapname for $map, sorry.");
        ( $maptype eq 'file' ) and ( $mapname !~ /^\// ) and
           $self->error("File mapname for type file not absolute ($mapname)");

        if ( $config->elementExists("$base/maps/$map/options") ) {
          $mpopts=$config->getValue("$base/maps/$map/options");
        } 

        my $etok;
        if ( &getValueDefault($config,"$base/maps/$map/enabled",'true') 
                ne 'false' ) {
          $etok="";
          if ( $maptype eq 'file' || $maptype eq 'direct' ) {
            $self->writeAutoMap($config,$mapname,"$base/maps/$map/entries") 
		or $etok="# Error in: ";
          } 
        } else {
          $etok="#";
        } 

        foreach my $mountp ( @mountpoints ) {
          $cnt+=NCM::Check::lines(
              "/etc/auto.master",
              linere => "^#?$mountp\\s+.*",
              goodre => "^$etok$mountp\\s+".$maptype.":".$mapname.
                              "\\s+".$mpopts."\\s*\$",
              good   => "$etok$mountp\t$maptype:$mapname\t$mpopts",
              keep   => "first",
              add    => "last");
        }
      }
    }

    #reload if changed the conf-file
    if($cnt) {
        $self->info("auto.master map modified, reloading autofs");
        unless (LC::Process::run('/sbin/service autofs reload')) {
        $self->error('command "/sbin/service autofs reload" failed');
        return; }
    }

    return 1;
}

1;      # Required for PERL modules

